var parseTypes = require('./Docstring__parseTypeString');

function TypeInfo(commentNode) {
  var description = commentNode.description;
  var name = (commentNode.name || '').trim();
  var typeInfo = {};

  var hasType = commentNode.type && commentNode.type.length > 0;

  // Example tags without a type but with a name:
  //
  //     /**
  //      * @example Do something.
  //      */
  //
  // Gives us:
  //
  //     {
  //       name: 'Do',
  //       description: 'something'
  //     }
  //
  // But we want:
  //
  //     {
  //       description: 'Do something.'
  //     }
  //
  if (commentNode.tag === 'example' && !hasType) {
    description = commentNode.source.replace(/^@example[ ]?/, '');

    if (commentNode.name.trim().length > 0) {
      var descriptionLines = description.split('\n');

      name = descriptionLines[0].trim();
      description = descriptionLines.slice(1).join('\n');
    }
  }

  if (name.length > 0) {
    typeInfo.name = name;
  }

  if (description && description.length > 0) {
    typeInfo.description = description;
  }

  if (hasType) {
    typeInfo.type = parseTypes(commentNode.type);

    if (!typeInfo.type || !typeInfo.type.name) {
      delete typeInfo.type;
    }
  }

  if (commentNode.optional) {
    typeInfo.isOptional = true;

    if (commentNode.default) {
      typeInfo.defaultValue = commentNode.default;
    }
  }

  return typeInfo;
}

module.exports = TypeInfo;
